@@ -244,7 +244,7 @@ int hydra_get_options(char *options[]) {
 	} else if (!strcmp(tmp, "sapr3")) {
 		widget = lookup_widget(GTK_WIDGET(wndMain), "spnSAPR3");
 		j = gtk_spin_button_get_value_as_int((GtkSpinButton *) widget);
-		snprintf(sapr3id, 10, "%d", j);
+		snprintf(sapr3id, sizeof(sapr3id), "%d", j);
 		options[i++] = "-m";
 		options[i++] = sapr3id;		
 
@@ -663,7 +663,7 @@ on_btnSave_clicked                     (GtkButton       *button,
 		
 		text = gtk_text_buffer_get_text(outputbuf, &start, &end, TRUE);
 
-		fd = open(filename, O_CREAT | O_TRUNC | O_WRONLY);
+		fd = open(filename, O_CREAT | O_TRUNC | O_WRONLY, 0644);
 		if ( fd > 0 ) {
 			write(fd, text, strlen(text));
 			close(fd);
