@@ -53,7 +53,7 @@ start_http(int s, unsigned long int ip, int port, unsigned char options, char *m
 */
 
   ptr = ((char *) index(buf, ' ')) + 1;
-  if (ptr != NULL && (*ptr == '2' || strncmp(ptr, "301", 3) == 0)) {
+  if (ptr != NULL && (*ptr == '2' || *ptr == '3' || strncmp(ptr, "403", 3) == 0 || strncmp(ptr, "404", 3) == 0)) {
     hydra_report_found_host(port, ip, "www", fp);
     hydra_completed_pair_found();
   } else {
