@@ -18,6 +18,8 @@ int alarm_went_off = 0;
 int use_ssl = 0;
 char ipaddr_str[INET_ADDRSTRLEN];
 int src_port=0;
+int __fck=0;
+int ssl_first=1;
 
 #ifdef LIBOPENSSL
 SSL *ssl = NULL;
@@ -145,7 +147,10 @@ internal__hydra_connect(unsigned long int host, int port, int protocol, int type
       printf("DEBUG_CONNECT_OK\n");
 
     if (use_proxy == 2) {
-      buf = malloc(4096);
+      if ((buf = malloc(4096)) == NULL) {
+        fprintf(stderr, "ERROR: could not malloc()\n");
+        return -1;
+      }
       memset(&target, 0, sizeof(target));
       memcpy(&target.sin_addr.s_addr, &host, 4);
       target.sin_family = AF_INET;
@@ -199,36 +204,44 @@ internal__hydra_connect_ssl(unsigned long int host, int port, int protocol, int
 {
   int socket, err;
 
-  /* XXX is this needed? */
-  SSL_load_error_strings();
-  SSLeay_add_ssl_algorithms();
-
-  /* context: ssl2 + ssl3 is allowed, whatever the server demands */
-  if ((sslContext = SSL_CTX_new(SSLv23_method())) == NULL) {
-    if (verbose) {
-      err = ERR_get_error();
-      fprintf(stderr, "SSL: Error allocating context: %s\n", ERR_error_string(err, NULL));
-    }
-    return -1;
+  if (ssl_first) {
+    SSL_load_error_strings();
+    SSL_library_init();	
+//    SSLeay_add_ssl_algorithms();
+    ssl_first = 0;
   }
-  /* set the compatbility mode */
-  SSL_CTX_set_options(sslContext, SSL_OP_ALL);
 
-  /* we set the default verifiers and dont care for the results */
-  (void) SSL_CTX_set_default_verify_paths(sslContext);
-  SSL_CTX_set_tmp_rsa_callback(sslContext, ssl_temp_rsa_cb);
-  SSL_CTX_set_verify(sslContext, SSL_VERIFY_NONE, NULL);
+  if (sslContext == NULL) {
+    /* context: ssl2 + ssl3 is allowed, whatever the server demands */
+    if ((sslContext = SSL_CTX_new(SSLv23_client_method())) == NULL) {
+      if (verbose) {
+        err = ERR_get_error();
+        fprintf(stderr, "SSL: Error allocating context: %s\n", ERR_error_string(err, NULL));
+      }
+      return -1;
+    }
+    /* set the compatbility mode */
+    SSL_CTX_set_options(sslContext, SSL_OP_ALL);
 
-  if ((socket = internal__hydra_connect(host, port, protocol, type)) < 0)
-    return -1;
+    /* we set the default verifiers and dont care for the results */
+    (void) SSL_CTX_set_default_verify_paths(sslContext);
+    SSL_CTX_set_tmp_rsa_callback(sslContext, ssl_temp_rsa_cb);
+    SSL_CTX_set_verify(sslContext, SSL_VERIFY_NONE, NULL);
+  }
 
   if ((ssl = SSL_new(sslContext)) == NULL) {
     if (verbose) {
       err = ERR_get_error();
       fprintf(stderr, "Error preparing an SSL context: %s\n", ERR_error_string(err, NULL));
     }
+    SSL_set_bio(ssl, NULL, NULL);
+    SSL_clear(ssl);
     return -1;
   }
+
+  if ((socket = internal__hydra_connect(host, port, protocol, type)) < 0)
+    return -1;
+
   SSL_set_fd(ssl, socket);
   if (SSL_connect(ssl) <= 0) {
     printf("ERROR %d\n", SSL_connect(ssl));
@@ -236,6 +249,7 @@ internal__hydra_connect_ssl(unsigned long int host, int port, int protocol, int
       err = ERR_get_error();
       fprintf(stderr, "Could not create an SSL session: %s\n", ERR_error_string(err, NULL));
     }
+    close(socket);
     return -1;
   }
 
@@ -276,11 +290,11 @@ void
 hydra_child_exit(int code)
 {
   if (code == 0)                /* normal quitting */
-    write(intern_socket, "Q", 1);
+    __fck=write(intern_socket, "Q", 1);
   if (code == 1)                /* no connect possible */
-    write(intern_socket, "C", 1);
+    __fck=write(intern_socket, "C", 1);
   if (code == 2)                /* application protocol error or service shutdown */
-    write(intern_socket, "E", 1);
+    __fck=write(intern_socket, "E", 1);
   exit(-1);
 }
 
@@ -294,7 +308,7 @@ char *
 hydra_get_next_pair()
 {
   if (pair[0] == 0) {
-    read(intern_socket, pair, sizeof(pair));
+    __fck=read(intern_socket, pair, sizeof(pair));
     if (memcmp(&HYDRA_EXIT, &pair, sizeof(HYDRA_EXIT)) == 0)
       return HYDRA_EXIT;
     if (pair[0] == 0)
@@ -327,7 +341,7 @@ hydra_get_next_password()
 void
 hydra_completed_pair()
 {
-  write(intern_socket, "N", 1);
+  __fck=write(intern_socket, "N", 1);
   pair[0] = 0;
 }
 
@@ -336,9 +350,9 @@ hydra_completed_pair_found()
 {
   char *login;
 
-  write(intern_socket, "F", 1);
+  __fck=write(intern_socket, "F", 1);
   login = hydra_get_next_login();
-  write(intern_socket, login, strlen(login) + 1);
+  __fck=write(intern_socket, login, strlen(login) + 1);
   pair[0] = 0;
 }
 
@@ -347,9 +361,9 @@ hydra_completed_pair_skip()
 {
   char *login;
 
-  write(intern_socket, "f", 1);
+  __fck=write(intern_socket, "f", 1);
   login = hydra_get_next_login();
-  write(intern_socket, login, strlen(login) + 1);
+  __fck=write(intern_socket, login, strlen(login) + 1);
   pair[0] = 0;
 }
 
@@ -437,6 +451,14 @@ hydra_connect_udp(unsigned long int host, int port)
 int
 hydra_disconnect(int socket)
 {
+#ifdef LIBOPENSSL
+  if (use_ssl && SSL_get_fd(ssl) == socket) {
+    /* SSL_shutdown(ssl); ...skip this--it slows things down */
+    SSL_set_bio(ssl, NULL, NULL);
+    SSL_clear(ssl);
+    use_ssl = 0;
+  }
+#endif
   close(socket);
   if (debug)
     printf("DEBUG_DISCONNECT\n");
@@ -492,7 +514,10 @@ hydra_receive_line(int socket)
   char buf[300], *buff;
   int i = 0, j = 0, k;
 
-  buff = malloc(sizeof(buf));
+  if ((buff = malloc(sizeof(buf))) == NULL) {
+    fprintf(stderr, "ERROR: could not malloc\n");
+    return NULL;
+  }
   memset(buff, 0, sizeof(buf));
 
   i = hydra_data_ready_timed(socket, (long) waittime, 0);
@@ -536,15 +561,16 @@ hydra_send(int socket, char *buf, int size, int options)
   if (debug) {
     int k;
     char *debugbuf = malloc(size + 1);
-
-    for (k = 0; k < size; k++)
-      if (buf[k] == 0)
-        debugbuf[k] = 32;
-      else
-        debugbuf[k] = buf[k];
-    debugbuf[size] = 0;
-    printf("DEBUG_SEND_BEGIN|%s|END\n", debugbuf);
-    free(debugbuf);
+    if (debugbuf != NULL) {
+      for (k = 0; k < size; k++)
+        if (buf[k] == 0)
+          debugbuf[k] = 32;
+        else
+          debugbuf[k] = buf[k];
+      debugbuf[size] = 0;
+      printf("DEBUG_SEND_BEGIN|%s|END\n", debugbuf);
+      free(debugbuf);
+    }
   }
 
 /*    if (hydra_data_ready_writing(socket)) < 1) return -1; XXX maybe needed in the future */
@@ -557,7 +583,7 @@ make_to_lower(char *buf)
   if (buf == NULL)
     return 1;
   while (buf[0] != 0) {
-    buf[0] = tolower(buf[0]);
+    buf[0] = tolower((int)buf[0]);
     buf++;
   }
   return 1;
