@@ -1,11 +1,20 @@
 
-/* mysql 3.2x.x to 5.0.x support - by mcbethh (at) u-n-f (dot) com */
+/* mysql 3.2x.x to 4.x support - by mcbethh (at) u-n-f (dot) com */
+/* david (dot) maciejak (at) gmail (dot) com for using libmysqlclient-dev, adding support for mysql version 5.x */
 
 #include <math.h>
 #include "hydra-mod.h"
 
+#define DEFAULT_DB "mysql"
+
+#ifndef LIBMYSQLCLIENT
 void hash_password(unsigned long *result, const char *password);
 char *scramble(char *to, const char *message, const char *password);
+#else
+
+#include <mysql/mysql.h>
+
+#endif
 
 extern int internal__hydra_recv(int socket, char *buf, int length);
 extern int hydra_data_ready_timed(int socket, long sec, long usec);
@@ -85,7 +94,9 @@ hydra_mysql_init(int sock)
   memcpy(salt, pos, 9);
 
   if (!strstr(server_version, "3.") && !strstr(server_version, "4.") && strstr(server_version, "5.")) {
+#ifndef LIBMYSQLCLIENT
     fprintf(stderr, "[Error] Not an MySQL protocol or unsupported version\n");
+#endif
     return 2;
   }
 
@@ -153,15 +164,46 @@ start_mysql(int sock, unsigned long int ip, int port, unsigned char options, cha
   char *response = NULL, *login = NULL, *pass = NULL;
   unsigned long response_len;
   char res = 0;
+  char database[256];
+  struct in_addr remote_ip;
 
   login = hydra_get_next_login();
   pass = hydra_get_next_password();
 
+  if(miscptr)
+      strncpy(database,miscptr,sizeof(database));
+  else
+      strncpy(database,DEFAULT_DB,sizeof(database));
+
+  remote_ip.s_addr = ip;
+
   /* read server greeting */
   res = hydra_mysql_init(sock);
   if (res == 2) {
+	/* old reversing protocol trick did not work */
+	/* try using the libmysql client if available*/
+	hydra_mysql_send_com_quit(sock);
+	sock = hydra_disconnect(sock);
+#ifdef LIBMYSQLCLIENT
+	MYSQL mysql;
+	mysql_init(&mysql);
+	/*mysql_options(&mysql,MYSQL_OPT_COMPRESS,0);*/
+	if (!mysql_real_connect(&mysql, inet_ntoa(remote_ip), login, pass, database, 0, NULL, 0))
+	{  
+	    if (debug)
+		    fprintf(stderr, "Failed to connect to database: Error: %s\n", mysql_error(&mysql));
+	    return 3;
+	}	
+	mysql_close(&mysql);
+    	hydra_report_found_host(port, ip, "mysql", fp);
+	hydra_completed_pair_found();
+	if (memcmp(hydra_get_next_pair(), &HYDRA_EXIT, sizeof(HYDRA_EXIT)) == 0)
+		return 3;
+    	return 1;
+#else    
     hydra_child_exit(2);
     exit(-1);
+#endif
   }
   if (res == 1)
     return 1;
@@ -255,6 +297,8 @@ service_mysql(unsigned long int ip, int sp, unsigned char options, char *miscptr
   }
 }
 
+#ifndef LIBMYSQLCLIENT
+
 /************************************************************************/
 
 /* code belowe is copied from mysql 3.23.57 source code (www.mysql.com) */
@@ -327,3 +371,5 @@ scramble(char *to, const char *message, const char *password)
   *to = 0;
   return to;
 }
+
+#endif
