@@ -1,8 +1,16 @@
+//
+//add support for AUTH PLAIN SMTP mechanism by <david dot maciejak at g m a i l dot com>
+//
+
 #include "hydra-mod.h"
 
+#define AUTH_LOGIN 1
+#define AUTH_PLAIN 2
+
 extern char *HYDRA_EXIT;
 
 char *buf;
+int auth_mechanism=AUTH_LOGIN;
 
 int
 start_smtpauth(int s, unsigned long int ip, int port, unsigned char options,
@@ -21,41 +29,69 @@ start_smtpauth(int s, unsigned long int ip, int port, unsigned char options,
       return (1);
     free(buf);
   }
-  sprintf(buffer, "AUTH LOGIN\r\n");
-  if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
-    return 1;
-  }
-  if ((buf = hydra_receive_line(s)) == NULL)
-    return 1;
-  if (strstr(buf, "334") == NULL) {
-    hydra_report(stderr, "Error: SMTP AUTH LOGIN error: %s\n", buf);
-    free(buf);
-    return 3;
-  }
-  free(buf);
+  if (verbose)
+	hydra_report(stderr, "Testing login: %s with password: %s\n", login ,pass);
+  if (auth_mechanism == AUTH_PLAIN) {
+     int buffer2_size;
+     sprintf(buffer, "AUTH PLAIN\r\n");
+     if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
+       return 1;
+     }
+     if ((buf = hydra_receive_line(s)) == NULL)
+       return 1;
+     if (strstr(buf, "334") == NULL) {
+       hydra_report(stderr, "Error: SMTP AUTH PLAIN error: %s\n", buf);
+       free(buf);
+       return 3;
+     }
+     free(buf);
 
-  sprintf(buffer2, "%.250s", login);
-  hydra_tobase64((unsigned char *) buffer2, strlen(buffer2), sizeof(buffer2));
-  sprintf(buffer, "%.250s\r\n", buffer2);
+     buffer2_size=strlen(login) + 1 + strlen(login) + 1 + strlen(pass);
+     memset(buffer2, 0, buffer2_size + 1);
+     strncpy(buffer2, login, strlen(login));
+     strncpy(buffer2 + strlen(login) + 1, login, strlen(login));
+     strncpy(buffer2 + strlen(login) + 1 + strlen(login) + 1, pass, strlen(pass));
+
+     hydra_tobase64((unsigned char *) buffer2, strlen(login)*2+strlen(pass)+2, sizeof(buffer2));
+     sprintf(buffer, "%.250s\r\n", buffer2);
+
+  } else {
+     /* by default trying AUTH LOGIN */
+     sprintf(buffer, "AUTH LOGIN\r\n");
+     if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
+       return 1;
+     }
+     if ((buf = hydra_receive_line(s)) == NULL)
+       return 1;
+     if (strstr(buf, "334") == NULL) {
+       hydra_report(stderr, "Error: SMTP AUTH LOGIN error: %s\n", buf);
+       free(buf);
+       return 3;
+     }
+     free(buf);
+     sprintf(buffer2, "%.250s", login);
+     hydra_tobase64((unsigned char *) buffer2, strlen(buffer2), sizeof(buffer2));
+     sprintf(buffer, "%.250s\r\n", buffer2);
+
+     if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
+       return 1;
+     }
+     if ((buf = hydra_receive_line(s)) == NULL)
+       return (1);
+     if (strstr(buf, "334") == NULL) {
+       hydra_report(stderr, "Error: SMTP AUTH LOGIN error: %s\n", buf);
+       free(buf);
+       return (3);
+     }
+     free(buf);
+
+     sprintf(buffer2, "%.250s", pass);
+     hydra_tobase64((unsigned char *) buffer2, strlen(buffer2), sizeof(buffer2));
+     sprintf(buffer, "%.250s\r\n", buffer2);
+   }
 
   if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
-    return 1;
-  }
-  if ((buf = hydra_receive_line(s)) == NULL)
-    return (1);
-  if (strstr(buf, "334") == NULL) {
-    hydra_report(stderr, "Error: SMTP AUTH LOGIN error: %s\n", buf);
-    free(buf);
-    return (3);
-  }
-  free(buf);
-
-  sprintf(buffer2, "%.250s", pass);
-  hydra_tobase64((unsigned char *) buffer2, strlen(buffer2), sizeof(buffer2));
-  sprintf(buffer, "%.250s\r\n", buffer2);
-
-  if (hydra_send(s, buffer, strlen(buffer), 0) < 0) {
-    return 1;
+       return 1;
   }
   if ((buf = hydra_receive_line(s)) == NULL)
     return (1);
@@ -79,11 +115,10 @@ void
 service_smtpauth(unsigned long int ip, int sp, unsigned char options,
 		 char *miscptr, FILE * fp, int port)
 {
-  int run = 1, next_run, sock = -1, resp = 0;
+  int run = 1, next_run, sock = -1, resp = 0, i = 0;
   int myport = PORT_SMTPAUTH, mysslport = PORT_SMTPAUTH_SSL;
   char *ptr;
 
-  /*  char *buffer = "EHLO mail.thc.org\r\n"; */
   char *buffer1 = "EHLO hydra\r\n";
   char *buffer2 = "HELO hydra\r\n";
 
@@ -95,7 +130,6 @@ service_smtpauth(unsigned long int ip, int sp, unsigned char options,
     case 1:			/* connect and service init function */
       if (sock >= 0)
 	sock = hydra_disconnect(sock);
-//      usleep(300000);
       if ((options & OPTION_SSL) == 0) {
 	if (port != 0)
 	  myport = port;
@@ -114,12 +148,6 @@ service_smtpauth(unsigned long int ip, int sp, unsigned char options,
 	hydra_child_exit(1);
       }
 
-//      while (hydra_data_ready(sock)) {
-//	if ((buf = hydra_receive_line(sock)) == NULL)
-//	  exit(-1);
-//	free(buf);
-//      }
-
       /* receive initial header */
       if ((buf = hydra_receive_line(sock)) == NULL)
 	exit(-1);
@@ -142,7 +170,7 @@ service_smtpauth(unsigned long int ip, int sp, unsigned char options,
       do {
         ptr = buf = hydra_receive_line(sock);
         if (buf != NULL) {
-          if (isdigit(buf[0]) && buf[3] == ' ')
+          if (isdigit((int)buf[0]) && buf[3] == ' ')
             resp = 1;
           else {
             if (buf[strlen(buf) - 1] == '\n')
@@ -151,15 +179,15 @@ service_smtpauth(unsigned long int ip, int sp, unsigned char options,
               buf[strlen(buf) - 1] = 0;
             if ((ptr = rindex(buf, '\n')) != NULL) {
               ptr++;
-              if (isdigit(*ptr) && *(ptr + 3) == ' ')
+              if (isdigit((int)*ptr) && *(ptr + 3) == ' ')
                 resp = 1;
             }
           }
         }
       } while (buf != NULL && resp == 0);
-//      buf != NULL && buf[3] != ' ' && strstr(buf, "250 ") == NULL);
+
       if (buf == NULL)
-        exit(-1); 
+        exit(-1);
      if (*ptr != '2') {
       if (hydra_send(sock, buffer2, strlen(buffer2), 0) < 0)
 	exit(-1);
@@ -167,7 +195,7 @@ service_smtpauth(unsigned long int ip, int sp, unsigned char options,
       do {
         ptr = buf = hydra_receive_line(sock);
         if (buf != NULL) {
-          if (isdigit(buf[0]) && buf[3] == ' ')
+          if (isdigit((int)buf[0]) && buf[3] == ' ')
             resp = 1;
           else {
             if (buf[strlen(buf) - 1] == '\n')
@@ -176,29 +204,35 @@ service_smtpauth(unsigned long int ip, int sp, unsigned char options,
               buf[strlen(buf) - 1] = 0;
             if ((ptr = rindex(buf, '\n')) != NULL) {
               ptr++;
-              if (isdigit(*ptr) && *(ptr + 3) == ' ')
+              if (isdigit((int)*ptr) && *(ptr + 3) == ' ')
                 resp = 1;
             }
           }
         }
       } while (buf != NULL && resp == 0);
-//      buf != NULL && buf[3] != ' ' && strstr(buf, "250 ") == NULL);
       if (buf == NULL)
         exit(-1); 
      }
-/*    old
-      if ((buf = hydra_receive_line(sock)) == NULL)
-	exit(-1);
-*/
-
-      /* check AUTH LOGIN supported */
-/*
-      if (strstr(buf, "LOGIN") == NULL) {
-        hydra_report(stderr, "Warning: SMTP AUTH LOGIN might not be supported: %s\n", buf);
+      if ((strstr(buf, "LOGIN") == NULL) && (strstr(buf, "PLAIN") != NULL)) {
+	auth_mechanism=AUTH_PLAIN;
       }
-*/
-      free(buf);
+      if ((miscptr != NULL)&& (strlen(miscptr) > 0)) {
+	      for (i = 0; i < strlen(miscptr); i++)
+       		miscptr[i] = (char) toupper(miscptr[i]);
 
+	       if (strncmp (miscptr,"LOGIN",5) == 0)
+ 		auth_mechanism=AUTH_LOGIN;
+
+	       if (strncmp (miscptr,"PLAIN",5) == 0)
+ 		auth_mechanism=AUTH_PLAIN;
+      }
+      if (verbose) {
+	if (auth_mechanism == AUTH_LOGIN)
+		hydra_report(stderr, "using SMTP AUTH LOGIN mechanism\n");
+	else
+		hydra_report(stderr, "using SMTP AUTH PLAIN mechanism\n");
+      }
+      free(buf);
       next_run = 2;
       break;
     case 2:			/* run the cracking function */
