@@ -41,8 +41,8 @@ int start_teamspeak(int s, unsigned long int ip, int port, unsigned char options
   teamspeak.clientlen = 9;
   strcpy((char *) &teamspeak.client, "TeamSpeak");
 
-  teamspeak.oslen = 0x1d;
-  strcpy((char *) &teamspeak.os, "Linux 2.6.9 #4 Tue Dec 21 15:");
+  teamspeak.oslen = 11;
+  strcpy((char *) &teamspeak.os, "Linux 2.6.9");
 
   memcpy(&teamspeak.misc, "\x02\x00\x00\x00\x20\x00\x3c\x00\x01\x02", 10);
 
