@@ -12,9 +12,7 @@ int no_line_mode;
 
 /* added extra null byte sending via "+ 1" below ... should work! */
 
-int
-start_telnet(int s, unsigned long int ip, int port, unsigned char options, char *miscptr, FILE * fp)
-{
+int start_telnet(int s, unsigned long int ip, int port, unsigned char options, char *miscptr, FILE * fp) {
   char *empty = "";
   char *login, *pass, buffer[300];
   int i = 0;
@@ -118,6 +116,7 @@ service_telnet(unsigned long int ip, int sp, unsigned char options, char *miscpt
 {
   int run = 1, next_run, sock = -1;
   int myport = PORT_TELNET, mysslport = PORT_TELNET_SSL;
+  int fck=0;
 
   hydra_register_socket(sp);
   if (memcmp(hydra_get_next_pair(), &HYDRA_EXIT, sizeof(HYDRA_EXIT)) == 0)
@@ -174,7 +173,7 @@ service_telnet(unsigned long int ip, int sp, unsigned char options, char *miscpt
 #ifdef PALM
             hydra_send(sock, "\xff\xfb\x22", 3, 0);
 #else
-            write(sock, "\xff\xfb\x22", 3);
+            fck=write(sock, "\xff\xfb\x22", 3);
 #endif
             first = 1;
           }
@@ -188,14 +187,14 @@ service_telnet(unsigned long int ip, int sp, unsigned char options, char *miscpt
 #ifdef PALM
             hydra_send(sock, buf2, 3, 0);
 #else
-            write(sock, buf2, 3);
+            fck=write(sock, buf2, 3);
 #endif
           } else if (buf2[1] == DO || buf2[1] == DONT) {
             buf2[1] = WONT;
 #ifdef PALM
             hydra_send(sock, buf2, 3, 0);
 #else
-            write(sock, buf2, 3);
+            fck=write(sock, buf2, 3);
 #endif
           }
           buf2 = buf2 + 3;
