@@ -1,5 +1,5 @@
 /*
- * hydra v5.0 (c) 2001-2006 by van Hauser / THC <vh@thc.org>
+ * hydra v5.9 (c) 2001-2010 by van Hauser / THC <vh@thc.org>
  * http://www.thc.org
  *
  * Parallized network login hacker. Do only use for legal purposes.
@@ -58,6 +58,15 @@ extern void service_smbnt(unsigned long int ip, int sp, unsigned char options, c
 #ifdef LIBSAPR3
 extern void service_sapr3(unsigned long int ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port);
 #endif
+#ifdef LIBFIREBIRD
+extern void service_firebird(unsigned long int ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port);
+#endif
+#ifdef LIBAFP
+extern void service_afp(unsigned long int ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port);
+#endif
+#ifdef LIBNCP
+extern void service_ncp(unsigned long int ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port);
+#endif
 #ifdef LIBSSH
 extern void service_ssh2(unsigned long int ip, int sp, unsigned char options, char *miscptr, FILE * fp, int port);
 #endif
@@ -67,7 +76,7 @@ extern void service_svn(unsigned long int ip, int sp, unsigned char options, cha
 
 /* ADD NEW SERVICES HERE */
 
-#define SERVICES  "telnet ftp pop3[-ntlm]   imap[-ntlm] smb smbnt http[s]-{head|get} http-{get|post}-form http-proxy cisco  cisco-enable vnc ldap2 ldap3 mssql mysql oracle-listener postgres nntp socks5   rexec rlogin pcnfs snmp rsh cvs svn icq sapr3 ssh2 smtp-auth[-ntlm] pcanywhere  teamspeak sip vmauthd"
+#define SERVICES  "telnet ftp pop3[-ntlm]   imap[-ntlm] smb smbnt http[s]-{head|get} http-{get|post}-form http-proxy cisco  cisco-enable vnc ldap2 ldap3 mssql mysql oracle-listener postgres nntp socks5   rexec rlogin pcnfs snmp rsh cvs svn icq sapr3 ssh2 smtp-auth[-ntlm] pcanywhere  teamspeak sip vmauthd firebird ncp afp"
 
 /* ADD NEW SERVICES HERE */
 
@@ -80,7 +89,7 @@ extern void service_svn(unsigned long int ip, int sp, unsigned char options, cha
 #define RESTOREFILE "./hydra.restore"
 
 #define PROGRAM   "Hydra"
-#define VERSION   "v5.4"
+#define VERSION   "v5.9"
 #define AUTHOR    "van Hauser / THC"
 #define EMAIL     "<vh@thc.org>"
 #define RESSOURCE "http://www.thc.org"
@@ -89,6 +98,8 @@ extern void service_svn(unsigned long int ip, int sp, unsigned char options, cha
 #define TASKS     16
 
 extern void hydra_tobase64(unsigned char *buf, int buflen, int bufsize);
+int fck=0;
+char *sck=NULL;
 
 /* some structure definitions */
 typedef struct {
@@ -206,7 +217,7 @@ char empty_login[2] = "";
 void
 help()
 {
-  printf("%s %s [%s] (c) 2006 by %s %s\n\n"
+  printf("%s %s [%s] (c) 2010 by %s %s\n\n"
          "Syntax: %s [[[-l LOGIN|-L FILE] [-p PASS|-P FILE]] | [-C FILE]] [-e ns]\n"
          " [-o FILE] [-t TASKS] [-M FILE [-T TASKS]] [-w TIME] [-f] [-s PORT] [-S] [-vV]\n" " server service [OPT]\n", PROGRAM, VERSION, RESSOURCE, AUTHOR, EMAIL, prg);
   printf("\nOptions:\n" "  -R        restore a previous aborted/crashed session\n");
@@ -306,25 +317,24 @@ hydra_restore_write(int print_msg)
     brain.targets = i;
     brain.ofp = NULL;
     brain.finished = brain.active = 0;
-    fwrite(&brain, sizeof(hydra_brain), 1, f);
-    fwrite(&hydra_options, sizeof(hydra_option), 1, f);
+    fck=fwrite(&brain, sizeof(hydra_brain), 1, f);
+    fck=fwrite(&hydra_options, sizeof(hydra_option), 1, f);
     if (hydra_options.outfile_ptr == NULL)
       fprintf(f, "\n");
     else
       fprintf(f, "%s\n", hydra_options.outfile_ptr);
     fprintf(f, "%s\n%s\n", hydra_options.miscptr, hydra_options.service);
-
 /*
     fprintf(f, "%lu\n", (unsigned long int) countservers);
     fprintf(f, "%lu\n", (unsigned long int) sizeservers);
-    fwrite(servers_ptr, sizeservers, 1, f);
+    fck=fwrite(servers_ptr, sizeservers, 1, f);
 */
-    fwrite(login_ptr, hydra_brains.sizelogin, 1, f);
+    fck=fwrite(login_ptr, hydra_brains.sizelogin, 1, f);
     if (hydra_options.colonfile == NULL || hydra_options.colonfile == empty_login)
-      fwrite(pass_ptr, hydra_brains.sizepass, 1, f);
+      fck=fwrite(pass_ptr, hydra_brains.sizepass, 1, f);
     for (j = 0; j < hydra_brains.targets; j++)
       if (hydra_targets[j]->done != 1) {
-        fwrite(hydra_targets[j], sizeof(hydra_target), 1, f);
+        fck=fwrite(hydra_targets[j], sizeof(hydra_target), 1, f);
         fprintf(f, "%s\n%d\n%d\n", hydra_targets[j]->target, hydra_targets[j]->login_ptr - login_ptr, hydra_targets[j]->pass_ptr - pass_ptr);
       }
     fprintf(f, "%s\n", PROGRAM);
@@ -353,7 +363,7 @@ hydra_restore_read()
     exit(-1);
   }
 
-  fgets(out, sizeof(out), f);
+  sck=fgets(out, sizeof(out), f);
   if (out[0] != 0 && out[strlen(out) - 1] == '\n')
     out[strlen(out) - 1] = 0;
   if (strcmp(out, PROGRAM) != 0) {
@@ -361,15 +371,15 @@ hydra_restore_read()
     exit(-1);
   }
 
-  fread(&hydra_brains, sizeof(hydra_brain), 1, f);
+  fck=(int)fread(&hydra_brains, sizeof(hydra_brain), 1, f);
   hydra_brains.ofp = stdout;
-  fread(&hydra_options, sizeof(hydra_option), 1, f);
+  fck=(int)fread(&hydra_options, sizeof(hydra_option), 1, f);
   hydra_options.restore = 1;
   verbose = hydra_options.verbose;
   debug = hydra_options.debug;
   waittime = hydra_options.waittime;
   port = hydra_options.port;
-  fgets(out, sizeof(out), f);
+  sck=fgets(out, sizeof(out), f);
   if (out[0] != 0 && out[strlen(out) - 1] == '\n')
     out[strlen(out) - 1] = 0;
   if (strlen(out) > 0) {
@@ -377,44 +387,44 @@ hydra_restore_read()
     strcpy(hydra_options.outfile_ptr, out);
   } else
     hydra_options.outfile_ptr = NULL;
-  fgets(out, sizeof(out), f);
+  sck=fgets(out, sizeof(out), f);
   if (out[0] != 0 && out[strlen(out) - 1] == '\n')
     out[strlen(out) - 1] = 0;
   hydra_options.miscptr = malloc(strlen(out) + 1);
   strcpy(hydra_options.miscptr, out);
-  fgets(out, sizeof(out), f);
+  sck=fgets(out, sizeof(out), f);
   if (out[0] != 0 && out[strlen(out) - 1] == '\n')
     out[strlen(out) - 1] = 0;
   hydra_options.service = malloc(strlen(out) + 1);
   strcpy(hydra_options.service, out);
   hydra_targets = malloc(hydra_brains.targets * sizeof(hydra_targets));
   login_ptr = malloc(hydra_brains.sizelogin);
-  fread(login_ptr, hydra_brains.sizelogin, 1, f);
+  fck=(int)fread(login_ptr, hydra_brains.sizelogin, 1, f);
   if (hydra_options.mode != 64) {
     pass_ptr = malloc(hydra_brains.sizepass);
-    fread(pass_ptr, hydra_brains.sizepass, 1, f);
+    fck=(int)fread(pass_ptr, hydra_brains.sizepass, 1, f);
   } else {                      /* colonfile mode */
     hydra_options.colonfile = empty_login;      /* dummy */
     pass_ptr = csv_ptr = login_ptr;
   }
   for (j = 0; j < hydra_brains.targets; j++) {
     hydra_targets[j] = malloc(sizeof(hydra_target));
-    fread(hydra_targets[j], sizeof(hydra_target), 1, f);
-    fgets(out, sizeof(out), f);
+    fck=(int)fread(hydra_targets[j], sizeof(hydra_target), 1, f);
+    sck=fgets(out, sizeof(out), f);
     if (out[0] != 0 && out[strlen(out) - 1] == '\n')
       out[strlen(out) - 1] = 0;
     hydra_targets[j]->target = malloc(strlen(out) + 1);
     strcpy(hydra_targets[j]->target, out);
-    fgets(out, sizeof(out), f);
+    sck=fgets(out, sizeof(out), f);
     hydra_targets[j]->login_ptr = login_ptr + atoi(out);
-    fgets(out, sizeof(out), f);
+    sck=fgets(out, sizeof(out), f);
     hydra_targets[j]->pass_ptr = pass_ptr + atoi(out);
     hydra_targets[j]->fail_count = 0;
     hydra_targets[j]->use_count = 0;
     if (hydra_targets[j]->max_use_count < 4)
       hydra_targets[j]->max_use_count = hydra_options.max_use;
   }
-  fgets(out, sizeof(out), f);
+  sck=fgets(out, sizeof(out), f);
   if (out[0] != 0 && out[strlen(out) - 1] == '\n')
     out[strlen(out) - 1] = 0;
   if (strcmp(out, PROGRAM) != 0) {
@@ -432,7 +442,7 @@ killed_childs(int signo)
   killed++;
   pid = wait3(NULL, WNOHANG, NULL);
   for(i=0;i<hydra_options.tasks;i++)	  {
-  	if(pid==hydra_arms[i]->pid)	{
+  	if (pid==hydra_arms[i]->pid)	{
   		hydra_kill_arm(i, 0, 0);
 		return;
 	}
@@ -650,7 +660,20 @@ hydra_spawn_arm(int arm_no, int target_no)
       if (strcmp(hydra_options.service, "postgres") == 0)
         service_postgres(hydra_targets[target_no]->ip, hydra_arms[arm_no]->sp[1], options, hydra_options.miscptr, hydra_brains.ofp, port);
 #endif
-if (strcmp(hydra_options.service, "pcanywhere") == 0)
+#ifdef LIBFIREBIRD
+      if (strcmp(hydra_options.service, "firebird") == 0)
+        service_firebird(hydra_targets[target_no]->ip, hydra_arms[arm_no]->sp[1], options, hydra_options.miscptr, hydra_brains.ofp, port);
+#endif
+#ifdef LIBAFP
+      if (strcmp(hydra_options.service, "afp") == 0)
+        service_afp(hydra_targets[target_no]->ip, hydra_arms[arm_no]->sp[1], options, hydra_options.miscptr, hydra_brains.ofp, port);
+#endif
+
+#ifdef LIBNCP
+      if (strcmp(hydra_options.service, "ncp") == 0)
+        service_ncp(hydra_targets[target_no]->ip, hydra_arms[arm_no]->sp[1], options, hydra_options.miscptr, hydra_brains.ofp, port);
+#endif
+      if (strcmp(hydra_options.service, "pcanywhere") == 0)
         service_pcanywhere(hydra_targets[target_no]->ip, hydra_arms[arm_no]->sp[1], options, hydra_options.miscptr, hydra_brains.ofp, port); 
       if (strcmp(hydra_options.service, "csv") == 0)
         service_cvs(hydra_targets[target_no]->ip, hydra_arms[arm_no]->sp[1], options, hydra_options.miscptr, hydra_brains.ofp, port);
@@ -678,15 +701,17 @@ if (strcmp(hydra_options.service, "pcanywhere") == 0)
         service_smtpauth_ntlm(hydra_targets[target_no]->ip, hydra_arms[arm_no]->sp[1], options, hydra_options.miscptr, hydra_brains.ofp, port);
       if (strcmp(hydra_options.service, "teamspeak") == 0)
         service_teamspeak(hydra_targets[target_no]->ip, hydra_arms[arm_no]->sp[1], options, hydra_options.miscptr, hydra_brains.ofp, port);
-            if (strcmp(hydra_options.service, "sip") == 0)
-                service_sip(hydra_targets[target_no]->ip, hydra_arms[arm_no]->sp[1], options, hydra_options.miscptr, hydra_brains.ofp, port);
+#ifdef LIBOPENSSL
+      if (strcmp(hydra_options.service, "sip") == 0)
+        service_sip(hydra_targets[target_no]->ip, hydra_arms[arm_no]->sp[1], options, hydra_options.miscptr, hydra_brains.ofp, port);
+#endif
 /* ADD NEW SERVICES HERE */
 
       /* just in case a module returns (which it shouldnt) we let it exit here */
       exit(0);
     } else {
       if (hydra_arms[arm_no]->pid > 0) {
-        write(hydra_arms[arm_no]->sp[1], "n", 1);       /* yes, a small "n" - this way we can distinguish later if the client successfully tested a pair and is requesting a new one or the mother did that */
+        fck=write(hydra_arms[arm_no]->sp[1], "n", 1);       /* yes, a small "n" - this way we can distinguish later if the client successfully tested a pair and is requesting a new one or the mother did that */
         fcntl(hydra_arms[arm_no]->sp[0], F_SETFL, O_NONBLOCK);
         hydra_arms[arm_no]->target_no = target_no;
         hydra_arms[arm_no]->active = 1;
@@ -756,6 +781,9 @@ hydra_lookup_port(char *service)
     {"snmp", PORT_SNMP, PORT_SNMP_SSL},
     {"cvs", PORT_CVS, PORT_CVS_SSL},
     {"svn", PORT_SVN, PORT_SVN_SSL},
+    {"firebird", PORT_FIREBIRD, PORT_FIREBIRD_SSL},
+    {"afp", PORT_AFP, PORT_AFP_SSL},
+    {"ncp", PORT_NCP, PORT_NCP_SSL},
     {"smtp-auth", PORT_SMTPAUTH, PORT_SMTPAUTH_SSL},
     {"smtp-auth-ntlm", PORT_SMTPAUTH, PORT_SMTPAUTH_SSL},
     {"teamspeak", PORT_TEAMSPEAK, PORT_TEAMSPEAK_SSL},
@@ -786,7 +814,7 @@ hydra_kill_arm(int arm_no, int killit, int fail)
   if (killit && hydra_arms[arm_no]->pid > 0)
     kill(hydra_arms[arm_no]->pid, SIGTERM);
   hydra_targets[hydra_arms[arm_no]->target_no]->use_count--;
-  if(hydra_arms[arm_no]->active)	{
+  if (hydra_arms[arm_no]->active)	{
      close(hydra_arms[arm_no]->sp[0]);
      close(hydra_arms[arm_no]->sp[1]);
   }
@@ -1004,7 +1032,7 @@ hydra_main(int soc, void *nessus, int argc, char *argv[])
       break;
 #endif
     case 'g':
-      fprintf(stderr, "Warning: option -g is deprected, ignored.\n");
+      fprintf(stderr, "Warning: option -g is deprecated, ignored.\n");
       break;
     case 'T':
       hydra_options.max_use = atoi(optarg);
@@ -1079,8 +1107,15 @@ hydra_main(int soc, void *nessus, int argc, char *argv[])
       fprintf(stderr, "[WARNING] The icq module is not working with the modern protocol version! (somebody else will need to fix this as I dont care for icq)\n");
       i = 1;
     }
-    if (strcmp(hydra_options.service, "mysql") == 0)
+    if (strcmp(hydra_options.service, "mysql") == 0) {
       i = 1;
+      if (hydra_options.tasks > 4) {
+        fprintf(stderr, "[INFO] Reduced number of tasks to 4 (mysql does not like many parallel connections)\n");
+        hydra_options.tasks = 4;
+        hydra_options.max_use = 4;
+      }
+      i = 1;
+}
     if (strcmp(hydra_options.service, "mssql") == 0)
       i = 1;
     if (strcmp(hydra_options.service, "oracle-listener") == 0) {
@@ -1092,6 +1127,30 @@ hydra_main(int soc, void *nessus, int argc, char *argv[])
       i = 1;
 #else
       bail("Compiled without LIBPOSTGRES support, module not available!");
+#endif
+    if (strcmp(hydra_options.service, "firebird") == 0)
+#ifdef LIBFIREBIRD
+      i = 1;
+#else
+      bail("Compiled without LIBFIREBIRD support, module not available!");
+#endif
+    if (strcmp(hydra_options.service, "afp") == 0)
+#ifdef LIBAFP
+      i = 1;
+#else
+      bail("Compiled without LIBAFP support, module not available!");
+#endif
+    if (strcmp(hydra_options.service, "svn") == 0)
+#ifdef LIBSVN
+      i = 1;
+#else
+      bail("Compiled without LIBSVN support, module not available!");
+#endif
+    if (strcmp(hydra_options.service, "ncp") == 0)
+#ifdef LIBNCP
+      i = 1;
+#else
+      bail("Compiled without LIBNCP support, module not available!");
 #endif
     if (strcmp(hydra_options.service, "pcanywhere") == 0)
       i = 1;
@@ -1161,8 +1220,8 @@ hydra_main(int soc, void *nessus, int argc, char *argv[])
       hydra_options.login = empty_login;
     }
         if (strcmp(hydra_options.service, "sip") == 0) {
-            if(hydra_options.miscptr == NULL) {
-                if(hydra_options.server != NULL) {
+            if (hydra_options.miscptr == NULL) {
+                if (hydra_options.server != NULL) {
                     hydra_options.miscptr = hydra_options.server;
                     i=1;
                 } else {
@@ -1453,7 +1512,7 @@ hydra_main(int soc, void *nessus, int argc, char *argv[])
   options = 0;
   if (hydra_options.ssl)
     options = options | OPTION_SSL;
-  printf("%s %s (c) 2006 by %s - use allowed only for legal purposes.\n", PROGRAM, VERSION, AUTHOR);
+  printf("%s %s (c) 2010 by %s - use allowed only for legal purposes.\n", PROGRAM, VERSION, AUTHOR);
   printf("%s (%s) starting at %s\n", PROGRAM, RESSOURCE, hydra_build_time());
   if (hydra_options.colonfile != NULL)
     printf("[DATA] %d tasks, %d servers, %lu login tries, ~%lu tries per task\n", hydra_options.tasks, hydra_brains.targets, hydra_brains.todo, math2);
@@ -1473,7 +1532,7 @@ hydra_main(int soc, void *nessus, int argc, char *argv[])
       perror("Error: Error creating outputfile:");
       exit(-1);
     }
-    fprintf(hydra_brains.ofp, "# %s %s run at %s on %s %s (%s ", PROGRAM, VERSION, hydra_build_time(), hydra_options.server, hydra_options.service, prg);
+    fprintf(hydra_brains.ofp, "# %s %s run at %s on %s %s (%s ", PROGRAM, VERSION, hydra_build_time(), hydra_options.server == NULL ? hydra_options.infile_ptr : hydra_options.server, hydra_options.service, prg);
     if (hydra_options.login != NULL)
       fprintf(hydra_brains.ofp, "-l %s ", hydra_options.login);
     if (hydra_options.pass != NULL)
@@ -1575,10 +1634,10 @@ hydra_main(int soc, void *nessus, int argc, char *argv[])
   /* this is the big function which starts the attacking children, feeds login/password pairs, etc.! */
   while (hydra_brains.finished < hydra_brains.targets && error == 0) {
     FD_ZERO(&fdreadarms);
-    for( arm_no=0, max_fd=1; arm_no < hydra_options.tasks; arm_no++) {
-      if(hydra_arms[arm_no]->active > 0 ) {
+    for (arm_no = 0, max_fd = 1; arm_no < hydra_options.tasks; arm_no++) {
+      if (hydra_arms[arm_no]->active > 0 ) {
 	 FD_SET(hydra_arms[arm_no]->sp[0],&fdreadarms);
-      if(max_fd<hydra_arms[arm_no]->sp[0])
+      if (max_fd<hydra_arms[arm_no]->sp[0])
         max_fd=hydra_arms[arm_no]->sp[0];
       }           
     }
@@ -1763,7 +1822,7 @@ hydra_main(int soc, void *nessus, int argc, char *argv[])
                 buflen = strlen(hydra_arms[arm_no]->current_login_ptr) + 1;
                 strcpy(buf + buflen, hydra_arms[arm_no]->current_pass_ptr);
                 buflen += strlen(hydra_arms[arm_no]->current_pass_ptr) + 1;
-                write(hydra_arms[arm_no]->sp[0], buf, buflen);
+                fck=write(hydra_arms[arm_no]->sp[0], buf, buflen);
                 if (debug || hydra_options.showAttempt) {
                     printf("[ATTEMPT] target %s - login \"%s\" - pass \"%s\" - child %d - %lu of %lu\n",
                            hydra_targets[target_no]->target, hydra_arms[arm_no]->current_login_ptr, hydra_arms[arm_no]->current_pass_ptr, arm_no,
@@ -1892,7 +1951,7 @@ hydra_main(int soc, void *nessus, int argc, char *argv[])
     unlink(RESTOREFILE);
     for (i = 0; i < hydra_options.tasks; i++)
       if (hydra_arms[i]->active > 0)
-        write(hydra_arms[i]->sp[0], HYDRA_EXIT, sizeof(HYDRA_EXIT));
+        fck=write(hydra_arms[i]->sp[0], HYDRA_EXIT, sizeof(HYDRA_EXIT));
     error = j = 0;
     while (error < MAXENDWAIT && j < hydra_options.tasks) {      /* hardcoded timeout of 15 seconds */
       j = 0;
