@@ -86,6 +86,12 @@
 #define PORT_SNMP_SSL   1993
 #define PORT_CVS        2401
 #define PORT_CVS_SSL    2401
+#define PORT_FIREBIRD   3050
+#define PORT_FIREBIRD_SSL 3050
+#define PORT_AFP	548
+#define PORT_AFP_SSL	548
+#define PORT_NCP        524
+#define PORT_NCP_SSL    524
 #define PORT_SVN        3690
 #define PORT_SVN_SSL    3690
 #define PORT_SMTPAUTH   25
