@@ -870,8 +870,8 @@ while (*s)
       s += skip;
     else
       {
-      if (islower(*s))
-	*s = toupper(*s);
+      if (islower((int)*s))
+	*s = toupper((int)*s);
       s++;
       }
     }
@@ -1116,7 +1116,7 @@ Contributed LGPL versions of some of the GPL'd Samba files.
 
 #define AddBytes(ptr, header, buf, count) \
 { \
-if (buf && count) \
+if (buf != NULL && count != 0) \
   { \
   SSVAL(&ptr->header.len,0,count); \
   SSVAL(&ptr->header.maxlen,0,count); \
