@@ -5,7 +5,7 @@
 #include <ctype.h>
 
 #define PROGRAM		"PW-Inspector"
-#define VERSION		"v0.1"
+#define VERSION		"v0.2"
 #define EMAIL		"vh@thc.org"
 #define WEB		"http://www.thc.org"
 
@@ -140,7 +140,7 @@ main(int argc, char *argv[])
         if (set_print) {
           j = 0;
           for (k = 0; k < strlen(buf); k++)
-            if (isprint(buf[k]) && isalnum(buf[k]) == 0)
+            if (isprint((int)buf[k]) != 0 && isalnum((int)buf[k]) == 0)
               j = 1;
           if (j)
             i++;
@@ -148,7 +148,7 @@ main(int argc, char *argv[])
         if (set_other) {
           j = 0;
           for (k = 0; k < strlen(buf); k++)
-            if (isprint(buf[k]) == 0 && isalnum(buf[k]) == 0)
+            if (isprint((int)buf[k]) == 0 && isalnum((int)buf[k]) == 0)
               j = 1;
           if (j)
             i++;
